
#define TIMEOUT (3000 / PERIODE_CAN)   // 3 s de silence d'un des moteurs = erreur

// Variables CAN
int32_t TempsSilenceNiveaux ;
int32_t TempsSilenceBMS ;
bool CANErrorNiveaux ;
bool CANErrorBMS ;

void CANMessagesSetup(void)
{
  msgJaugeage.IDE = CAN_ID_STD;
  msgJaugeage.RTR = CAN_RTR_DATA;
  msgJaugeage.ID = COMMANDE_JAUGEAGE_ID ;
  msgJaugeage.DLC = 1;
  msgJaugeage.Data[0] = 0x1;    // Intervalle standard 1 h
}

// Scrutation de la boîte aux lettres
// interprétation des trames reçues
// On ignore la trame si mise à jour des paramètres en cours
void ReceptionCAN(void)
{
  int32_t i ;
  
  int16_t Pr = 0 ;
  CanMsg *r_msg;

//  while ((r_msg = canBus.recv()) != NULL)
  while ( ( r_msg = can_rx_queue_get() ) != NULL )
  {
    // On récupère les données retournées par les deux moteurs  
    // Niveaux  
    switch ( r_msg->ID )
    {
      case NIVEAUX_RESERVOIR_ID :
        TempsSilenceNiveaux = 0 ;              // Remise à zéro du compteur à chaque trame reçue
        CANErrorNiveaux = false ;
  
        for ( i = 0 ; i < 4 ; i++ )
          Contenus[i] = ((uint16_t)r_msg->Data[2*i]) | (((uint16_t)r_msg->Data[(2*i)+1]) << 8) ;
        break ;
    
      case BMS_SOC_ID :
        TempsSilenceBMS = 0 ;              // Remise à zéro du compteur à chaque trame reçue
        CANErrorBMS = false ;
        Pr = r_msg->Data[0] << 8 ;
        Pr |= r_msg->Data[1] ;
        SOCx10 = Pr / 10 ;                      // Le SOC est en 1/10000
        Pr = r_msg->Data[2] << 8 ;                
        Pr |= r_msg->Data[3] ;    
        SOHx10 = Pr / 10 ;                      // Le SOH est en 1/10000 -- mais non implemente' !
        break ;

      case MOTEURS_ID :
        // Si la version passerelle est 30 ou plus, c'est elle qui génère l'estimation du SOC
        Pr = r_msg->Data[6] ;
        Pr |= r_msg->Data[7] << 8 ;
        SOCx10P = Pr ;
        break ;
        
     case BMS_TEMPERATURES_ID :
        Pr = r_msg->Data[4] << 8 ;
        Pr |= r_msg->Data[5] << 0 ;
        TemperatureBatterie = Pr ;
        break ;
        
      default :
        break ;
    }

    can_rx_queue_free() ;
  }

  TempsSilenceNiveaux++ ; 
  if ( TempsSilenceNiveaux > TIMEOUT )
  {
    TempsSilenceNiveaux = TIMEOUT + 1 ;            // Pour éviter le débordement
    CANErrorNiveaux = true ;
  }
  
  TempsSilenceBMS++ ; 
  if ( TempsSilenceBMS > TIMEOUT )
  {
    TempsSilenceBMS = TIMEOUT + 1 ;            // Pour éviter le débordement
    CANErrorBMS = true ;
  }
}
