// Options for compilation : Generic board STM32F1, reference F103RBT6, UART enabled, USB support none, optimize default, runtime library default, upload method maple dfu bootloader original.
// Affichage des niveaux reçus par CAN sur afficheur 2 x 16 
// Ajout bouton poussoir pour provoquer le jaugeage. Implique utilisation programme Jauge3 dans la jauge
// Activer le define pour imprimer (debug)
// Version 3 : modification syntaxe commande de jaugeage
// Version 5 : recompilation avec librairie CAN corrigée
// Version 6 : ajout affichage données batterie, en alternance avec les réservoirs
// depuis version 6 compilation sur noyau à date de novembre 2018
// Version 7 : affichage SOC selon BMS et aussi selon Passerelle

//#define PRINT_RECU
//#define PRINT_EMIS
//#define SERIAL

#include <Olimexino.h>
#include <string.h>
#include <IWatchdog.h>
#include <LiquidCrystal.h>
#include <CANIdentifiers.h> // Description de la messagerie
#include "stm32F1xx_hal_can.h"

#undef SERIAL   // Cet identificateur est défini avant ce point !
//#define PORT_SERIE_USB    // Attention : si activé, le CAN sera inhibé !
#define PORT_SERIE_PHYSIQUE

#ifdef PORT_SERIE_USB
#define SERIAL Serial   // Pour serial USB
#endif

#ifdef PORT_SERIE_PHYSIQUE
#define SERIAL Serial2   // Port physique sur D0 et D1
#endif

#define TAILLE_TAMPON 164

#define LED_JAUNE D3                 // Numero de sortie pour la led jaune de la carte olimexino (voyant d'erreur)
#define LED_VERTE          D13      // Numéro de sortie pour la led verte de la carte olimexino

#define WDR25S 4000                 // Valeur de rechargement du watchdog pour 25.6 s
#define LED_CARTE D13      // Numéro de sortie pour la led
#define STACK_SIZE 400              // Valeur proche du maximum admissible
#define PERIODE_CAN 5         // ms
#define PERIODE_RAFRAICHISSEMENT_AFFICHEUR 2500 // ms
#define DIVISEUR (PERIODE_RAFRAICHISSEMENT_AFFICHEUR / PERIODE_CAN)
#define ATTENTE_JAUGEAGE 46000L
#define DIVISEUR_ATTENTE (ATTENTE_JAUGEAGE / PERIODE_CAN)
#define REPOS 10
#define PERIODE_CLIGNOTEMENT 1000      // ms
#define CAPACITE_GAZOLE 627            // L
#define CAPACITE_EAU_POTABLE 627       // L
#define CAPACITE_EAUX_GRISES 627       // L
#define CAPACITE_EAUX_NOIRES 627       // L
#define LCD_RS D8          // Affectation des ports pour l'afficheur
#define LCD_RW D9
#define LCD_E D10
#define LCD_D0 D4
#define LCD_D1 D5
#define LCD_D2 D6
#define LCD_D3 D7
#define BOUTON D15          // Bouton poussoir de demande de jaugeage
#define NB_LIGNES 2      // 2            // Dimensions de l'afficheur
#define NB_COLONNES 16
#define SP 0x20
#define ANTIREBOND_BOUTON 500  // ms
#define TIMER_POMPE_CHARGE 3
#define CANAL_POMPE_CHARGE 1
#define PIN_POMPE_CHARGE D12
//#define PERIODE_POMPE_CHARGE 200     // µs
#define DUREE_JAUGEAGE 46000L
#define FREQUENCE_POMPE_CHARGE      5000      // 5 kHz

typedef struct
{
  uint32_t ID;    // CAN ID
  uint8_t IDE;    // CAN_ID_STD for standard and CAN_ID_EXT for extended
  uint8_t RTR;
  uint8_t DLC;
  uint8_t Data[8];
  uint8_t FMI;
} CanMsg;

LiquidCrystal lcd(LCD_RS, LCD_RW, LCD_E, LCD_D0, LCD_D1, LCD_D2, LCD_D3) ;      // LCD utilise tout les port Analogique
//HardwareCAN canBus(CAN1_BASE);
HardwareTimer *PompeDeCharge;          // Pour produire la polarisation négative de l'afficheur
uint32_t PompeDeChargeChannel ;

bool Clignoteur ;
// Réservoirs
int16_t Contenus[4] ;
int32_t DebounceButton ;
CanMsg msgJaugeage;
int32_t Diviseur ;
int32_t DiviseurAttente ;

// Variables du BMS
uint16_t SOCx10 = 0xFFFF ;                  // SOC BMS pour mille ; FFFF = invalide
uint16_t SOCx10P = 0xFFFF ;                 // SOC passerelle pour mille ; FFFF = invalide
uint16_t SOHx10 = 1000 ;                    // pour mille
int16_t TemperatureBatterie = 250 ;              // en 1/10 °C

// Port série pour debug. Si physique, doit être explicitement créé
#ifdef PORT_SERIE_PHYSIQUE
  HardwareSerial Serial2(PA3, PA2) ;
#endif

// Signalement d'erreur. Procédure terminale, fait clignoter LED CARTE. Séquence de N points puis espace.
void Error(int32_t N)
{
  int32_t i ;
  
  while (1)
    {
      for ( i = 0 ; i < N ; i++ )
      {
        digitalWrite(LED_CARTE,HIGH);
        delay(100);
        digitalWrite(LED_CARTE,LOW);
        delay(200);
      }
      delay(300);
    }
}


void setup() {

  pinMode(LED_JAUNE, OUTPUT);
  pinMode(BOUTON, INPUT_PULLUP) ;
//  pinMode(PIN_POMPE_CHARGE, PWM);
  digitalWrite(LED_JAUNE,HIGH);
#ifdef SERIAL
  SERIAL.begin(115200);
  delay(2000);
#endif  
// Timer PWM 
  TIM_TypeDef *Instance = (TIM_TypeDef *)pinmap_peripheral(digitalPinToPinName(PIN_POMPE_CHARGE), PinMap_PWM);
  PompeDeChargeChannel = STM_PIN_CHANNEL(pinmap_function(digitalPinToPinName(PIN_POMPE_CHARGE), PinMap_PWM));
  // Instantiate HardwareTimer object. Thanks to 'new' instantiation, HardwareTimer is not destructed when setup() function is finished.
  PompeDeCharge = new HardwareTimer(Instance);

  PompeDeCharge->pause();
  PompeDeCharge->setPWM(PompeDeChargeChannel, PIN_POMPE_CHARGE, FREQUENCE_POMPE_CHARGE, 50) ;   // Initialisation à 5 kHz, 50%
  PompeDeCharge->resume();

#ifndef PORT_SERIE_PHYSIQUE
  CANSetup() ;
  CANMessagesSetup() ;
#endif
  delay(1000) ;

#ifdef SERIAL
  Serial.println("Ready !");
#endif  
  IWatchdog.begin(WDR25S);          // Initialisation du chien de garde, nourri par la tâche Affichage

  SetupAFfichage() ;
  digitalWrite(LED_JAUNE, LOW);

 }

void loop() 
{
#ifndef SERIAL
  CommunicationCAN() ;
#endif

  if ( DiviseurAttente > 0 )    // Temporisation du message "jaugeage en cours" après appui bouton
  {
    DiviseurAttente-- ; 
  }
  
  if ( Diviseur >= DIVISEUR )
  {
    Diviseur = 0 ;
    Affichage() ;
  }
  
  Diviseur++ ;
  
  if ( digitalRead(BOUTON) == LOW )
    DebounceButton++ ;
  else
    DebounceButton = 0 ;
    
  if ( DebounceButton == ( ANTIREBOND_BOUTON / PERIODE_CAN ) )
  {
    DiviseurAttente = DIVISEUR_ATTENTE ;    // Lancement durée masquage jaugeage -- affichage message attente
    CANsend(&msgJaugeage) ;      // Appui bouton : envoi ordre jaugeage
    ReinitAFfichage() ;
    while ( digitalRead(BOUTON) == LOW ) ;    // attente relâchement bouton
    Diviseur = 0 ;
  }
    
  delay(PERIODE_CAN) ;
}
  

  
  // Communication CAN
void CommunicationCAN(void)
{
  ReceptionCAN() ;  

  // Si erreur, tentative de récupération
//  if ( canBus.Port->ESR & CAN_ESR_BOFF )
//    CANSetup() ; 
  
    IWatchdog.reload() ;                // Nourrir le chien de garde
 }
