// Identificateurs des trames CAN et description de leur structure

// #define BMS_CAN_STANDARD // Définir pour utiliser les identificateurs standard, sinon, etendus 


#define COMMANDE_BARRE_ID_BABORD 100        // 64h  Commandes vers verin direction babord
#define COMMANDE_BARRE_ID_TRIBORD 101       // 65h  Commande vers verin direction tribord
#define COMMANDE_RELEVAGE_ID_BABORD 102     // 66h  Commandes vers verin relevage babord
#define COMMANDE_RELEVAGE_ID_TRIBORD 103    // 67h  Commandes vers verin relevage tribord
#define RETOUR_POSITION_ID_BABORD 104       // 68h  Position verin direction babord vers poste pilotage
#define RETOUR_POSITION_ID_TRIBORD 105      // 69h  Position verin direction tribord vers poste pilotage
#define PARAMETRES_ID_BABORD 108            // 6Ch  Parametres vers commande verins babord
#define PARAMETRES_ID_TRIBORD 109           // 6Dh  Parametres vers commande verins tribord

#define GYRO_ID 112                         // 70h  Donnee du gyroscope et du GPS
#define MOTEURS_ID 114                      // 72h  Regime et puissance moteur babord
#define GROUPE_ID 116                       // 74h  Etat du groupe electrogene
#define COMMANDE_MOTEURS_ID 118             // 76h  Commande (position joystick imposee) babord et tribord

#define NIVEAUX_RESERVOIR_ID 120            // 78h  Niveaux des 4 reservoirs vers affichage interieur
#define COMMANDE_JAUGEAGE_ID 122            // 7Ah  Intervalle de jaugeage, ou forcage jaugeage immediat vers jauges
#define COMMANDE_CONVECTEURS_ID 124         // 7Ch  Commande confort ou hors gel des convecteurs, vers jauges
#define TEMPERATURES_SECTEUR_ID  150        // 96h  Temperatures lues et presence secteur par boitier jauges
#define POULS_POSTE_PILOTAGE_ID 200        // C8h  Emis chaque seconde pour prise de controle par poste pilotage normal

// Cas d'utilisation directe de la connexion CAN du BMS. Les identifiants sont standard sauf keep alive.
#ifdef BMS_CAN_STANDARD
#define BMS_TENSIONS_ID 0x1                  // Total, element min et element max
#define BMS_COURANTS_ID 0x2                  // Courant
#define BMS_TEMPERATURES_ID 0x3              // Temp max et min des sondes, et moyenne
#define BMS_SOC_ID     0x5                   // Egalement le SOH, qui n'est pas implemente actuellement
#define BMS_DISJ_ID    0x7                   // Tous les etats, dont le disjoncteur
#else
#define BMS_TENSIONS_ID 0x18FF01D0           // Total, element min et element max
#define BMS_COURANTS_ID 0x18FF02D0
#define BMS_TEMPERATURES_ID 0x18FF03D0       // Temp max et min des sondes, et moyenne
#define BMS_SOC_ID     0x18FF05D0            // Egalement le SOH, qui n'est pas implemente actuellement
#define BMS_DISJ_ID    0x18FF07D0            // Tous les etats, dont le disjoncteur
#endif

#define BMS_KEEP_ALIVE 0x180AD000            // Keep alive

#define CODE_RAZ_GYRO 0xC5					// Code qui force la raz de la vitesse de giration	

/*    STRUCTURE DES TRAMES

COMMANDE_BARRE_ID_BABORD 0x64
COMMANDE_BARRE_ID_TRIBORD 0x65

De timonerie vers vérins
DLC = 2
Octets 0 et 1 : Commande direction (-500, 500), - à babord

************************************************
COMMANDE_RELEVAGE_ID_BABORD 0x66
COMMANDE_RELEVAGE_ID_TRIBORD 0x67

De timonerie vers vérins
DLC = 2
Octets 0 et 1 : Commande relevage (0, 1000), 0 relevé, 1000 baissé

************************************************

RETOUR_POSITION_ID_BABORD 0x68
RETOUR_POSITION_ID_TRIBORD 0x69

De vérins ver timonerie
DLC = 6
Octets 0 et 1 : Position réelle direction (-500, 500), - à babord
Octet 2 : Centre (-128, 127), - à babord
Octet 3 : Butée basse (0, 255), 0 ne limitant pas la course
Octet 4 : Butée haute (0, 255), 0 ne limitant pas la course
Octet 5 : 
          bit 0 : Local (1 = local) 
          bit 1 : Faute direction (1 = faute) 
          bit 2 : Faute relevage (1 = faute) 
          bit 3 : Mouvement Relevage Terminé (1 = Terminé) 

************************************************

PARAMETRES_ID_BABORD 0x6C
PARAMETRES_ID_TRIBORD 0x6D

De timonerie vers vérins
DLC = 4

Octet 0 : Centre (-128, 127), - à babord
Octet 1 : Butée basse (0, 255), 0 ne limitant pas la course
Octet 2 : Butée haute (0, 255), 0 ne limitant pas la course
Octet 3 : Doit être à 1 pour valider la trame

************************************************

GYRO_ID 0x70
De passerelle vers Timonerie V3
DLC = 7

Octet 0 : partie faible de la vitesse de giration brute
Octet 1 : partie forte de la vitesse de giration brute
Octet 2 : partie faible de la vitesse de giration calibrée
Octet 3 : partie forte de la vitesse de giration calibrée
Octet 4 : vitesse au fond, en 1/10 km/h
octet 5 : 0 = GPS inactif ; autre : GPS actif
octet 6 : 0 = Gyroscope ok ; autre : erreur gyroscope

************************************************

MOTEURS_ID 0x72
De passerelle vers Timonerie V3
DLC = 6

Octet 0 : partie faible du regime babord (t/min, signe)
Octet 1 : partie forte du regime babord
Octet 2 : puissance babord (1/10 kW, non signe) (0xFF = non connecte)
Octet 3 : partie faible du regime tribord (t/min, signe)
Octet 4 : partie forte du regime tribord
Octet 5 : puissance tribord (1/10 kW, non signe) (0xFFFF = non connecte)
Octet 6 : partie faible du SOC intégré par la passerelle
Octet 7 : partie forte du SOC intégré par la passerelle

************************************************

GROUPE_ID 0x74

De passerelle vers Timonerie V3
DLC = 8
Octet 0 : partie faible de la puissance (1/10 kW, non signe) (0xFFFF = non connecte)
Octet 1 : partie forte de la puissance 
Octet 2 : etat du groupe : 0 = inconnu, 1 = demarrage, 2 = marche, 3 = en regulation, 4=arret en cours, 5=arrete
Octet 3 : etat HDS remote :  0=HDS_NON_CONNECTE, 1=HDS_EN_COURS, 2=HDS_CONNECTE, 3=HDS_TELECOMMANDE, 4=HDS_COMM_ERREUR, 
          5=HDS_ERREUR_CONECTE, 6=HDS_ERREUR_TELECOMMANDE, 7=HDS_REPRISE_JOYSTICK, 8=HDS_INCONNU
Octet 4 : partie faible du numéro d'erreur
Octet 5 : partie forte du numéro d'erreur
octet 6 : type d'ecran d'erreur (choix des boutons a afficher)
Octet 7 : numéro de version du firmware PasserelleTimonerie

************************************************

COMMANDE_MOTEURS_ID 0x76

De Timonerie V3 vers passerelle
DLC = 8
Octet 0 : partie faible du joystick babord (0-2000)
Octet 1 : partie forte du joystick babord
Octet 2 : partie faible du joystick tribord (0-2000)
Octet 3 : partie forte du joystick tribord
octet 4 : commande du groupe (0 = repos, 1 = demande demarrage, 2 = demande arret)
octet 5 : commande de la trompe (0 = arret, autre = marche)
octet 6 : bouton en reponse a une erreur (ok, yes, no, suppress, ignore)
octet 7 : commande de remise à zéro de la vitesse de giration (active si valeur = CODE_RAZ_GYRO)

************************************************

NIVEAUX_RESERVOIR_ID 0x78

De capteurs de niveau à indicateur LCD et routeur
DLC = 8

Octets 0 et 1 : Volume Gazole
Octets 2 et 3 : Volume Eau potable
Octets 4 et 5 : Volume Eaux grises
Octets 6 et 7 : Volume Eaux noires

************************************************
COMMANDE_JAUGEAGE_ID 0x7A

De IndicateurLCDJauges à capteurs de niveau
DLC = 1
Octet 0 : Durée prochain intervalle entre lectures, en heures (0 à 255) ; 0 = jamais
La réception d'une trame avec DLC a zero lance la mesure

************************************************

COMMANDE_CONVECTEURS_ID 0x7C
De routeur a boitier jauge
DLC = 3
Octet 0 : Carre
Octet 1 : Chambre prop.
Octet 2 : Chambre amis
0 = confort, autre : hors gel.

************************************************

TEMPERATURES_SECTEUR_ID 0x96
De capteurs de niveau a routeur avant
DLC = 5

Octets 0 et 1 : Temperature exterieure en 1/10°
Octets 2 et 3 : Temperature chambre des machines en 1/10°
Octet 4 : 0 = absence secteur ; autre = présence secteur.

************************************************
POULS_POSTE_PILOTAGE_ID 0xC8 -- Le flux de ce message (timeout 1 s) demande la connexion au C3 Cockpit
Emis par poste de pilotage chaque seconde pour signaler qu'il a la main
DLC = 0

*/
